<?php

/**
 * This file is part of the SOLURIS - RGPD Management application.
 *
 * (c) Donovan Bourlard <donovan.bourlard@outlook.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

declare(strict_types=1);

namespace App\Tests\Utils;

trait ReflectionTrait
{
    /**
     * Call protected/private method of a class.
     *
     * @param object &$object    Instantiated object that we will run method on
     * @param string $methodName Method name to call
     * @param array  $parameters array of parameters to pass into method
     *
     * @throws \ReflectionException
     *
     * @return mixed method return
     */
    protected function invokeMethod(&$object, string $methodName, array $parameters = [])
    {
        $reflection = new \ReflectionClass(\get_class($object));
        $method     = $reflection->getMethod($methodName);
        $method->setAccessible(true);

        return $method->invokeArgs($object, $parameters);
    }

    /**
     * Call protected/private static method of a class.
     *
     * @param string $class      Class name that we will run method on
     * @param string $methodName Method name to call
     * @param array  $parameters array of parameters to pass into method
     *
     * @throws \ReflectionException
     *
     * @return mixed method return
     */
    protected function invokeStaticMethod(string $class, string $methodName, array $parameters = [])
    {
        $reflection = new \ReflectionClass($class);
        $method     = $reflection->getMethod($methodName);
        $method->setAccessible(true);

        return $method->invokeArgs(null, $parameters);
    }

    /**
     * Sets a protected property on a given object via reflection.
     *
     * @param $object - instance in which protected value is being modified
     * @param $property - property on instance being modified
     * @param $value - new value of the property being modified
     *
     * @throws \ReflectionException
     */
    protected function setProtectedProperty(&$object, $property, $value)
    {
        $reflectionClass    = new \ReflectionClass(\get_class($object));
        $reflectionProperty = $reflectionClass->getProperty($property);
        $reflectionProperty->setAccessible(true);
        $reflectionProperty->setValue($object, $value);
    }

    /**
     * Sets a protected method on a given object via reflection.
     *
     * @param $object - instance in which protected value is being modified
     * @param $method - property on instance being modified
     * @param $value - new value of the property being modified
     *
     * @throws \ReflectionException
     *
     * @return \ReflectionClass
     */
    protected function setProtectedMethod(&$object, $method, $value)
    {
        $reflectionClass    = new \ReflectionClass(\get_class($object));
        $reflectionProperty = $reflectionClass->getMethod($method);
        $reflectionProperty->setAccessible(true);
        $reflectionProperty->setValue($object, $value);

        return $reflectionClass;
    }
}
